@@ -2,7 +2,7 @@ import copy
 import operator
 import textwrap
 from collections import OrderedDict as odict
-from typing import (  # pylint: disable=unused-import
+from typing import (
     Any,
     Dict,
     Iterator,
@@ -177,7 +177,7 @@ class Orderer:
     def rule_weight(self, row, rule, regexp_key):
         return len(set(row).intersection(set(rule["attrs"][regexp_key].pattern))) / len(row)
 
-    def get_order(self, row, cmd_direct):
+    def get_order(self, row, cmd_direct, scope: str | None = None):
         f_order = None
         f_weight = 0
         f_rule = ""
@@ -186,6 +186,15 @@ class Orderer:
         block_exit = platform.VENDOR_EXIT[self.vendor]
 
         for (order, (raw_rule, rule)) in enumerate(ordering.items()):
+            if (
+                (rule_scope := rule["attrs"]["scope"]) is not None
+                and scope not in rule_scope
+            ):
+                continue
+
+            if rule["attrs"]["global"]:
+                children.append((raw_rule, rule))
+
             direct_matched = bool(rule["attrs"]["direct_regexp"].match(row))
             if not rule["attrs"]["order_reverse"] and (direct_matched or rule["attrs"]["reverse_regexp"].match(row)):
                 # если не указано order_reverse - правило считается прямым
@@ -395,7 +404,7 @@ def make_patch(pre, rb, hw, add_comments, orderer=None, _root_pre=None, do_commi
         for (key, diff) in content["items"].items():
             # чтобы logic не мог поменять атрибуты
             rule_pre = content.copy()
-            attrs = rule_pre["attrs"].copy()
+            attrs = copy.deepcopy(rule_pre["attrs"])
 
             iterable = attrs["logic"](
                 rule=attrs,
@@ -416,7 +425,7 @@ def make_patch(pre, rb, hw, add_comments, orderer=None, _root_pre=None, do_commi
                             patch_row = "%s %s" % (row, comments)
 
                     # pylint: disable=unused-variable
-                    (order, order_direct, ordering, order_rule) = orderer.get_order(row, direct)
+                    (order, order_direct, ordering, order_rule) = orderer.get_order(row, direct, scope="patch")
                     fmt_row = patch_row
                     # fmt_row += "  # %s" % str(order_rule)  # uncomment to debug ordering
 
@@ -544,8 +553,7 @@ def _select_match(matches, rules):
         for (rule, is_cr_allowed) in map(operator.itemgetter(0), matches):
             if is_cr_allowed:
                 local_children = merge_dicts(local_children, rule["children"]["local"])
-            # optional break on is_cr_allowed==False?
-
+                # optional break on is_cr_allowed==False?
                 global_children = merge_dicts(global_children, rule["children"]["global"])
 
     global_children = merge_dicts(global_children, rules["global"])
@@ -555,9 +563,10 @@ def _select_match(matches, rules):
         "global": global_children,
     }
 
-    match = {"attrs": f_rule["attrs"]}
+    match = {"attrs": copy.deepcopy(f_rule["attrs"])}
     match.update(f_other)
-    return (match, children_rules)
+
+    return match, children_rules
 
 
 def _rules_local_global(rules):
