@@ -10,14 +10,16 @@ __all__ = [
     "RDFilterFilterGenerator",
     "RDFilter",
     "IpPrefixList",
+    "IpPrefixListMember",
     "PrefixListFilterGenerator",
     "get_policies",
+    "ip_prefix_list",
 ]
 
 from .aspath import AsPathFilterGenerator
 from .community import CommunityListGenerator
 from .cumulus_frr import CumulusPolicyGenerator
-from .entities import CommunityList, AsPathFilter, CommunityType, CommunityLogic, RDFilter, IpPrefixList
+from .entities import CommunityList, AsPathFilter, CommunityType, CommunityLogic, RDFilter, IpPrefixList, IpPrefixListMember, ip_prefix_list
 from .execute import get_policies
 from .policy import RoutingPolicyGenerator
 from .prefix_lists import PrefixListFilterGenerator
