@@ -1,4 +1,4 @@
-from annet.rpl_generators import AsPathFilter, CommunityList, CommunityType, RDFilter, IpPrefixList
+from annet.rpl_generators import AsPathFilter, CommunityList, CommunityType, RDFilter, ip_prefix_list
 
 AS_PATH_FILTERS = [
     AsPathFilter("ASP_EXAMPLE", [".*123456.*"]),
@@ -16,6 +16,6 @@ RD_FILTERS = [
 ]
 
 PREFIX_LISTS = [
-    IpPrefixList("IPV6_LIST_EXAMPLE", ["2a13:5941::/32"]),
-    IpPrefixList("IPV4_LIST_EXAMPLE", ["0.0.0.0/8", "10.0.0.0/8"]),
+    ip_prefix_list("IPV6_LIST_EXAMPLE", ["2a13:5941::/32"]),
+    ip_prefix_list("IPV4_LIST_EXAMPLE", ["0.0.0.0/8", "10.0.0.0/8"]),
 ]
